/// <reference types="node" />
import { EventEmitter } from 'events';
import createRequestClient from '../create-request-client';
import { JSONLikeObject, JSONObject } from '../json-object';
import { InputData, Features } from '../mapping-kit';
import type { DynamicFieldResponse, InputField, RequestExtension, ExecuteInput, Result, SyncModeDefinition, DynamicFieldContext } from './types';
import type { JSONSchema4 } from 'json-schema';
import { AuthTokens } from './parse-settings';
import { Logger, StatsContext, TransactionContext, StateContext, DataFeedCache } from './index';
declare type MaybePromise<T> = T | Promise<T>;
declare type RequestClient = ReturnType<typeof createRequestClient>;
export declare type RequestFn<Settings, Payload, Return = any, AudienceSettings = any, ActionHookInputs = any> = (request: RequestClient, data: ExecuteInput<Settings, Payload, AudienceSettings, ActionHookInputs>) => MaybePromise<Return>;
export interface BaseActionDefinition {
    title: string;
    description: string;
    platform?: 'cloud' | 'web';
    defaultSubscription?: string;
    hidden?: boolean;
    fields: Record<string, InputField>;
}
declare type HookValueTypes = string | boolean | number | Array<string | boolean | number>;
declare type GenericActionHookValues = Record<string, HookValueTypes>;
declare type GenericActionHookBundle = {
    [K in ActionHookType]?: {
        inputs?: GenericActionHookValues;
        outputs?: GenericActionHookValues;
    };
};
export interface ActionDefinition<Settings, Payload = any, AudienceSettings = any, GeneratedActionHookBundle extends GenericActionHookBundle = any> extends BaseActionDefinition {
    dynamicFields?: {
        [K in keyof Payload]?: Payload[K] extends object ? {
            [ObjectProperty in keyof Payload[K] | '__keys__' | '__values__']?: RequestFn<Settings, Payload[K], DynamicFieldResponse, AudienceSettings>;
        } : RequestFn<Settings, Payload, DynamicFieldResponse, AudienceSettings>;
    };
    perform: RequestFn<Settings, Payload, any, AudienceSettings>;
    performBatch?: RequestFn<Settings, Payload[], any, AudienceSettings>;
    hooks?: {
        [K in ActionHookType]?: ActionHookDefinition<Settings, Payload, AudienceSettings, NonNullable<GeneratedActionHookBundle[K]>['outputs'], NonNullable<GeneratedActionHookBundle[K]>['inputs']>;
    };
    syncMode?: SyncModeDefinition;
}
export declare const hookTypeStrings: readonly ["onMappingSave", "retlOnMappingSave"];
export declare type ActionHookType = typeof hookTypeStrings[number];
export interface ActionHookResponse<GeneratedActionHookOutputs> {
    successMessage?: string;
    savedData?: GeneratedActionHookOutputs;
    error?: {
        message: string;
        code: string;
    };
}
export interface ActionHookDefinition<Settings, Payload, AudienceSettings, GeneratedActionHookOutputs, GeneratedActionHookTypesInputs> {
    label: string;
    description: string;
    inputFields?: Record<string, Omit<InputField, 'dynamic'> & {
        dynamic?: RequestFn<Settings, Payload, DynamicFieldResponse, AudienceSettings, GeneratedActionHookTypesInputs>;
    }>;
    outputTypes?: Record<string, {
        label: string;
        description: string;
        type: string;
        required: boolean;
    }>;
    performHook: RequestFn<Settings, Payload, ActionHookResponse<GeneratedActionHookOutputs>, AudienceSettings, GeneratedActionHookTypesInputs>;
}
export interface ExecuteDynamicFieldInput<Settings, Payload, AudienceSettings = any> {
    settings: Settings;
    audienceSettings?: AudienceSettings;
    payload: Payload;
    page?: string;
    auth?: AuthTokens;
    features?: Features | undefined;
    statsContext?: StatsContext | undefined;
    hookInputs?: GenericActionHookValues;
    dynamicFieldContext?: DynamicFieldContext;
}
interface ExecuteBundle<T = unknown, Data = unknown, AudienceSettings = any, ActionHookValues = any> {
    data: Data;
    settings: T;
    audienceSettings?: AudienceSettings;
    mapping: JSONObject;
    auth: AuthTokens | undefined;
    hookOutputs?: Record<ActionHookType, ActionHookValues>;
    features?: Features | undefined;
    statsContext?: StatsContext | undefined;
    logger?: Logger | undefined;
    dataFeedCache?: DataFeedCache | undefined;
    transactionContext?: TransactionContext;
    stateContext?: StateContext;
}
export declare class Action<Settings, Payload extends JSONLikeObject, AudienceSettings = any> extends EventEmitter {
    readonly definition: ActionDefinition<Settings, Payload, AudienceSettings>;
    readonly destinationName: string;
    readonly schema?: JSONSchema4;
    readonly hookSchemas?: Record<string, JSONSchema4>;
    readonly hasBatchSupport: boolean;
    readonly hasHookSupport: boolean;
    private extendRequest;
    constructor(destinationName: string, definition: ActionDefinition<Settings, Payload, AudienceSettings>, extendRequest?: RequestExtension<Settings, any>);
    execute(bundle: ExecuteBundle<Settings, InputData | undefined, AudienceSettings>): Promise<Result[]>;
    executeBatch(bundle: ExecuteBundle<Settings, InputData[], AudienceSettings>): Promise<Result[]>;
    private extractFieldContextAndHandler;
    executeDynamicField(field: string, data: ExecuteDynamicFieldInput<Settings, Payload, AudienceSettings>, dynamicFn?: RequestFn<Settings, Payload, DynamicFieldResponse, AudienceSettings>): Promise<DynamicFieldResponse>;
    executeHook(hookType: ActionHookType, data: ExecuteInput<Settings, Payload, AudienceSettings>): Promise<ActionHookResponse<any>>;
    private performRequest;
    private createRequestClient;
    private afterResponse;
    private parseResponse;
}
export {};
